import { evaluateFetchApiArgs, fetchApi, isErrorResponse, isAppRoute, getRouteQuery } from '@ts-rest/core';
import { useQuery, useInfiniteQuery, useMutation } from '@tanstack/vue-query';

const queryFn = (route, clientArgs, argsMapper) => {
    return async (queryFnContext) => {
        const args = typeof argsMapper === 'function'
            ? argsMapper(queryFnContext)
            : argsMapper;
        const fetchApiArgs = evaluateFetchApiArgs(route, clientArgs, args);
        const result = await fetchApi({
            ...fetchApiArgs,
            fetchOptions: {
                ...((queryFnContext === null || queryFnContext === void 0 ? void 0 : queryFnContext.signal) && { signal: queryFnContext.signal }),
                ...fetchApiArgs.fetchOptions,
            },
        });
        // If the response is not a 2XX, throw an error to be handled by react-query
        if (isErrorResponse(result)) {
            throw result;
        }
        return result;
    };
};

const getRouteUseQuery = (route, clientArgs) => {
    return (queryKey, args, options) => {
        const dataFn = queryFn(route, clientArgs, args);
        return useQuery({ queryKey, queryFn: dataFn, ...options });
    };
};

const getRouteUseInfiniteQuery = (route, clientArgs) => (queryKey, argsMapper, options) => {
    const dataFn = async (context) => {
        const resultingQueryArgs = argsMapper(context);
        const innerDataFn = queryFn(route, clientArgs, resultingQueryArgs);
        return innerDataFn(undefined);
    };
    return useInfiniteQuery({
        queryKey,
        queryFn: dataFn,
        ...options,
    });
};

const getRouteUseMutation = (route, clientArgs) => (options) => {
    const mutationFunction = async (args) => {
        const dataFn = queryFn(route, clientArgs, args);
        return dataFn(undefined);
    };
    return useMutation({
        mutationFn: mutationFunction,
        ...options,
    });
};

const initQueryClient = (router, args) => Object.fromEntries(Object.entries(router).map(([key, subRouter]) => isAppRoute(subRouter)
    ? [
        key,
        {
            query: getRouteQuery(subRouter, args),
            mutation: getRouteQuery(subRouter, args),
            useQuery: getRouteUseQuery(subRouter, args),
            useInfiniteQuery: getRouteUseInfiniteQuery(subRouter, args),
            useMutation: getRouteUseMutation(subRouter, args),
        },
    ]
    : [key, initQueryClient(subRouter, args)]));

export { getRouteUseMutation, initQueryClient };
